PRO L1C
; PROGRAM L1C
;
VERSION = '30OCT07'
; 30OCT07  AD  Fix 1 point offset in calculating cloud index (l1c_cldidx.pro)
; 15AUG07  AD  Comment out output directory in BATCHMODE - just use local dir.
; 21MAY07  AD  Apply FREE_LUN to LUNLST also
; 17MAY07  AD  Bug fix: ensure L1BFIL defined as string for batch mode
; 02MAR07  AD/CP Allow for 'batch mode' to process sequence of L1B files    
; 19JAN07  AD  Bug fix: add MA mode to list of recognised modes (29 sweeps)
; 28JUL06  AD  Add nominal altitudes of more 0.0625 modes
;              Fix bug in l1c_cldidx
; 15JUN05  AD  Correct order of 27 sweep nominal altitudes
; 23MAY05  AD  v2.0 Add spectrum type and resolution to output.
;              allow for nominal altitudes for 27 sweeps as well as 17
; 14MAR05  AD  Correction: allow for corrupt bands in L1C_INPMDS
; 28FEB05  AD  Correction: use ALT argument in L1C_SWPMIC if not nom.alts
; 20DEC04  AD  L1C_WRTMIC: Limit NESR to 99999 in output files 
; 16DEC04  AD  Allow for scan# > 99 in output filename
;              Add warning messages for corrupt bands
; 07DEC04  AD  v1.5 Use nominal tan.hts for assigning MWs to sweeps if nom.scan
;              And replace ALT_ERR field with ALT_NOM - L1C format 1.5
; 02DEC04  AD  Add switch RESFAC to reduce spectral resolution if required
; 08NOV04  AD  Tidied up and simplified, renamed from lv2pp.pro to l1c.pro
; 29SEP04  AD  Change NPTS format from 4X,I4 to X,I7 to allow full bands as MWs
; 02FEB04  ABB Change format string to bring length < 80 characters
; 16DEC03  CP  If band A is corrupted, Cloud ratio of band D MWs 
; 15DEC03  CP  For missing sweeps set Cloud Ratio equal zero
; 17NOV03  ABB v1.4 Write Cloud Ratio (Spang et al. 2003)
; 17FEB03  AD  Remove parameters.pro setting of constants
; 09OCT02  AD  v1.3 Write Cloud channel radiance 
; 26AUG02  AD  v1.2 Write radiances in free format instead of 8F10.4
; 13AUG02  AD  v1.1 Modified date format yyyymmdd instead of yymmdd
; 23JUL02  CP  Modified sweep.pro (correction of one point shift)
; 02JUL02  CP  Extract MW for nominal altitude (introduction of ALTNOM)
;          AD  Use SZALST rather than SZA (correct calculation of LST)
; 14JUN02  AD  Rename to lv2pp (was realv1) and add !path definition
; 04JUN02  AD  Modified to free LUN if incomplete scan is found
;              Correct addressing of Scan Information ADS
;              Restructure to loop through scans, sweeps
; 06MAY02  CP  Original

;
; DESCRIPTION
; Program to extract and apodise MW spectra from MIPAS L1B file
; Also extracted for sweep: ALT,ALT_ERR,LAT,LON,RAD_EARTH 
; Apodize the full L1B spectrum 
;
; When run, it first checks if there is a file 'l1bfil.list' containing a list
; of L1B file to be processed. If this exists then the program runs in 
; 'batch mode' without any user response, taking each file in turn, looking for
; a local file 'mwlist.lst' containing the microwindows to be used (this 
; filename is specified in 'l1c_inpmic.pro') and processing every scan,
; creating/writing the output to directories ./yyyymmdd/ooooo (oooo is orbit#).
;
; If 'l1bfil.list' doesn't exist, then it runs interactively, prompting the
; user for the name of the L1B file, a file containing the list of microwindows
; and the first,last scans to be processed. Output files are in the local
; directory 


; Normally, RESFAC=1.
; Set RESFAC .LT. 1 if the spectral resolution is to be reduced (ie the
; sampling interval increased) for output. 
; The NESR will also be scaled appropriately (*sqrt(RESFAC))

RESFAC = 1.0D0

IF RESFAC NE 1.0D0 THEN PRINT, $
  'W-L1C: Reducing spectral resolution, factor=', RESFAC

; If the output file format is changed, modify this real number
; (written as first non-comment record in output files)
; Changing this to 1.5 will produce v1.5 output format
L1CFMT = 2.0

; Set IDL path to search directory containing other procedures
!PATH = '/home/crun/eodg/mipas/l1c:' + !PATH

; List of Band names
LSTBND = [ 'A', 'AB', 'B', 'C', 'D' ]

; Set list of nominal altitudes (assuming 17 or 27 sweeps)
; 17 Sweeps = Nominal mode, full resolution
N17SWP = [68,60,52,47,42,39,36,33,30,27,24,21,18,15,12,9,6]
; 27 Sweeps = Nominal mode, reduced resolution
N27SWP = [70,66,62,58,54,50,46,43,40,37,34,31,29,27,25,23,21,19.5,18,16.5,$
          15,13.5,12,10.5,9,7.5,6]
; 18 Sweeps = Old UTLS-1 
N18SWP = [49,44,39,34,31,28,26,24,22,20.5,19,17.5,16,14.5,13,11.5,10,8.5]
; 19 Sweeps = New UTLS-1 
N19SWP = [49,44.5,40,35.5,31,28,25,23,21,19,17.5,16,14.5,13,11.5,10,8.5,7,5.5]
; 11 Sweeps = UTLS-2
N11SWP = [42,37,33,29,26,23,20,18,16,14,12]
; 29 Sweeps = MA
N29SWP = [102,99,96,93,90,87,84,81,78,75,72,69,66,63,60,57,54,51,48,45,42,39,$
          36,33,30,27,24,21,18]
; 35 Sweeps = UA
N35SWP = [172,167,162,157,152,147,142,137,132,127,122,117,112,107,102,99,96,$
          93,90,87,84,81,78,75,72,69,66,63,60,57,54,51,48,45,42]
; 25 Sweeps = NLC
N25SWP = [102,99,96,93,90,87,85.5,84,82.5,81,79.5,78,75,72,69,66,63,60,57,54,$
          51,48,45,42,39]
; 12 Sweeps = AE
N12SWP = [38,33.5,29,24.5,20,17,15,13,11.5,10,8.5,7]

FILSAV = 'l1c_l1bfil.save'

; Decide if this is to be run in 'Batch' Mode or interactively for 1 L1B file
; Older versions of IDL may not have the function "FILE_TEST", in which case
; see notes in README.txt
BATCHMODE = FILE_TEST ( 'l1bfil.list' )

L1BFIL = ''
IF BATCHMODE THEN OPENR, LUNLST, 'l1bfil.list', /GET_LUN ELSE BEGIN
; Single L1B interactively
  IF FILE_TEST ( FILSAV ) THEN $
    PRINT, 'L1B file [<CR>= use ' + FILSAV + ']', FORMAT='($,A)' $
    ELSE PRINT, 'L1B file', FORMAT='($,A)' 
  READ, L1BFIL, FORMAT='(A)'
  IF L1BFIL NE '' THEN BEGIN               ; user gives L1B filename
    OPENW, LUNLST, FILSAV, /GET_LUN        ; so save in l1c_l1bfil.save
    PRINTF, LUNLST, L1BFIL, FORMAT='(A)'
    FREE_LUN, LUNLST
  ENDIF
  OPENR, LUNLST, FILSAV, /GET_LUN          ; read filename from l1c_l1bfil.save
ENDELSE
; At this point, either l1bfil.list or l1c_l1bfil.save is opened as LUNLST
; and contains name(s) the L1B file(s) to be processed

WHILE NOT EOF ( LUNLST ) DO BEGIN    ; Loop over L1B files

  READF, LUNLST, L1BFIL, FORMAT='(A)'
  OPENR, LUNL1B, L1BFIL, /SWAP_ENDIAN, /GET_LUN

; Extract nominal date from filename (assume positioned 47 chars from end)
; eg 'MIP_NL__1PODPA20030218_012100_000060272014_00003_05067_0865.N1'
;                   12345678901234567890123456789012345678901234567
  DATE = STRMID ( L1BFIL, 47, 8, /REVERSE)

; If in batchmode, check if directory exists for date, else create one
; comment out output directory - just use local directory in BATCHMODE
;  IF BATCHMODE AND NOT FILE_TEST ( DATE, /DIRECTORY ) THEN $
;    SPAWN, 'mkdir ' + DATE

; Extract data from L1B Main Product Header 
  L1C_INPMPH, LUNL1B, ABS_ORBIT, SPH_SIZE

; Set up output directory for Batch mode 
  IF BATCHMODE THEN BEGIN
; comment out output directory - just use local directory in BATCHMODE
;    OUTDIR = './' + DATE + '/' + STRING ( ABS_ORBIT, FORMAT='(I5.5)' ) + '/'
;    IF NOT FILE_TEST ( OUTDIR ) THEN SPAWN, 'mkdir ' + OUTDIR
     outdir = ''
  ENDIF ELSE OUTDIR = ''

; Extract data from L1B Specific Product Header
  L1C_INPSPH, LUNL1B, SPH_SIZE, TOT_SWEEPS, TOT_SCANS, NUM_SWEEPS_PER_SCAN, $
    NUM_POINTS_PER_BAND, FIRST_WAVENUM, LAST_WAVENUM, NUM_NESR_PNTS,$
    NESR_FIRST_WAVENUM, NESR_LAST_WAVENUM, SCAN_ADS_OFFSET, $
    MDS_OFFSET, MDS_SIZE   

; Calculate nominal spectral resolution from A band information
  RESL1B = ( LAST_WAVENUM[0] - FIRST_WAVENUM[0] ) $
         / ( NUM_POINTS_PER_BAND[0] - 1 )
  RESL1C = RESL1B / RESFAC

  CASE NUM_SWEEPS_PER_SCAN OF
    11: PRINT, 'I-L1C: Identified as RR UTLS-2 mode
    12: PRINT, 'I-L1C: Identified as RR AE mode
    17: PRINT, 'I-L1C: Identified as FR NOM mode
    18: PRINT, 'I-L1C: Identified as RR old UTLS-1 mode
    19: PRINT, 'I-L1C: Identified as RR new UTLS-1 mode
    25: PRINT, 'I-L1C: Identified as RR NLC mode
    27: PRINT, 'I-L1C: Identified as RR NOM mode
    29: PRINT, 'I-L1C: Identified as RR MA mode
    35: PRINT, 'I-L1C: Identified as RR UA mode
  ELSE: PRINT, 'W-L1C: Unable to identify obs.mode from L1B'
  ENDCASE

; Get list of Microwindows 
  L1C_INPMIC, FIRST_WAVENUM, LAST_WAVENUM, RESL1C, BATCHMODE, $
    NMIC, CODMIC, BNDMIC, WN1MIC, WN2MIC, HT1MIC, HT2MIC

; Calculate number of points in full spectrum starting at 0cm-1
  NSPC = ROUND ( LAST_WAVENUM[4] / RESL1B ) + 1

; Calculate Norton Beer Apodisation factors for zero-filled spectrum
  L1C_NBFAC, NSPC, RESFAC, NSPCZF, NBFAC

; Calculate spacing of NESR tabulation
  DWNON = ( NESR_LAST_WAVENUM - NESR_FIRST_WAVENUM ) / ( NUM_NESR_PNTS - 1 ) 

; Ask for first and last scans to be used (default=use all)
  ISCAN1 = 1
  ISCAN2 = TOT_SCANS
  IF NOT BATCHMODE THEN BEGIN
    PRINT, 'First,Last Scan Nos [<CR>=1,' + $
      STRING ( TOT_SCANS, FORMAT='(I3)' ) + ']', FORMAT='($,A)'
    ANSWER = ''
    READ, ANSWER
    IF ANSWER NE '' THEN READS, ANSWER, ISCAN1, ISCAN2
    IF ISCAN1 LT 1 OR ISCAN2 GT TOT_SCANS OR ISCAN2 LT ISCAN1 THEN $
      MESSAGE, 'F-L1C: Illegal Scan# value(s)'
  ENDIF

; If the first output scan is not the first scan in the file, still need to
; read through all the SCAN ADS records sequentially since the start of each
; record depends on the length of the previous record

  ADS_OFFSET = SCAN_ADS_OFFSET
  ISWEEP = 1L
  FOR ISCAN = 1, ISCAN2 DO BEGIN
  
    L1C_INPSCAN, LUNL1B, NUM_NESR_PNTS, ADS_OFFSET, ADS_LENGTH, NUM_SWEEPS, $
      SCAN_TIME, NESR_DATA
    ADS_OFFSET = ADS_OFFSET + ADS_LENGTH

    IF ISCAN LT ISCAN1 THEN GOTO, NEXT_SCAN  ; just step through SCAN ADS recs

    PRINT, 'I-L1C: Processing Scan#', ISCAN

; Check that number of sweeps is the nominal value for this scan
    IF NUM_SWEEPS NE NUM_SWEEPS_PER_SCAN THEN BEGIN
      PRINT, 'I-L1C: Skip scan - No.Sweeps = ', NUM_SWEEPS, $
             ' NE Nominal No.Sweeps = ', NUM_SWEEPS_PER_SCAN 
      GOTO, NEXT_SCAN
    ENDIF

; Open L1C file for this scan and write header records
    IF ISCAN LE 99 THEN STRSCN = STRING ( ISCAN, FORMAT='(I2.2)' ) ELSE $
      STRSCN = STRING ( ISCAN, FORMAT='(I3.3)' ) 
    OUTFIL = OUTDIR + 'L1C_' + STRING( ABS_ORBIT, FORMAT='(I5.5)') + $
             '_SCAN#' + STRSCN
    PRINT, 'I-L1C: Creating File ' + OUTFIL
    OPENW, LUNL1C, OUTFIL, /GET_LUN
    L1C_WRTHDR, LUNL1C, VERSION, L1CFMT, L1BFIL, NUM_SWEEPS_PER_SCAN, RESL1C, $
      RESFAC

; Find Sweep# matching Scan time to position at start of nominal scan
    L1C_SWPTIM, LUNL1B, SCAN_TIME, TOT_SWEEPS, MDS_OFFSET, MDS_SIZE, ISWEEP

    FOR ISWP = 0, NUM_SWEEPS-1 DO BEGIN  ; loop over sweeps within scan

; Extract sweep information
      L1C_INPMDS, LUNL1B, ISWEEP, MDS_OFFSET, MDS_SIZE, NUM_POINTS_PER_BAND,$
        FIRST_WAVENUM, LAST_WAVENUM, RESL1B, NSPC, MDS_TIME, ALT, $
        ALT_ERR, LAT, LON, RAD_EARTH, BAND_VAL, SPEC 

; Warn if any corrupt bands
      FOR IBND = 0, 4 DO BEGIN
        IF BAND_VAL[IBND] NE 0 THEN PRINT, 'W-L1C: Sweep#' + $ 
          STRING ( ISWP+1, FORMAT='(I2.2)' ) + ' (' + $
          STRING ( ALT, FORMAT='(F4.1)' ) + 'km) ' + LSTBND[IBND] + $
          ' Band Corrupt, Band_Val=', BAND_VAL[IBND]
      ENDFOR

; Determine nominal altitude
      CASE NUM_SWEEPS_PER_SCAN OF
        11: ALT_NOM = N11SWP(ISWP)  ; RR UTLS-2
        12: ALT_NOM = N12SWP(ISWP)  ; RR AE
        17: ALT_NOM = N17SWP(ISWP)  ; FR NOM
        18: ALT_NOM = N18SWP(ISWP)  ; RR old UTLS-1
        19: ALT_NOM = N19SWP(ISWP)  ; RR new UTLS-1
        25: ALT_NOM = N25SWP(ISWP)  ; RR NLC
        27: ALT_NOM = N27SWP(ISWP)  ; RR NOM
        29: ALT_NOM = N29SWP(ISWP)  ; RR MA
        35: ALT_NOM = N35SWP(ISWP)  ; RR UA
      ELSE: ALT_NOM = 0 ; cannot identify scan pattern, so no nominal altitude
      ENDCASE

; Determine which microwindows will be output for this tangent height
; Change ALT_NOM to ALT if MW selection based on reported altitude is preferred
      IF ALT_NOM EQ 0 THEN  $
        L1C_SWPMIC, ALT, BAND_VAL, NMIC, BNDMIC, HT1MIC, HT2MIC, NMWSWP, $
          OUTMIC  $
      ELSE $
        L1C_SWPMIC, ALT_NOM, BAND_VAL, NMIC, BNDMIC, HT1MIC, HT2MIC, NMWSWP, $
          OUTMIC

; Calculate Cloud index from unapodized spectrum
      L1C_CLDIDX, SPEC, RESL1B, BAND_VAL, CLDIDX

; Apodization of the spectrum 
      IF NMIC GT 0 THEN L1C_APOD, NSPCZF, NBFAC, RESFAC, NSPC, SPEC

; Calculate Cloud Channel radiance from apodized spectrum
; Note that CLDRAD will use unapodised spectral values if NMIC=0, ie no
; microwindow list is supplied, in order to save time
      L1C_CLDRAD, SPEC, RESL1C, BAND_VAL, CLDRAD

; Write out Sweep Header information (time, location, etc)
      L1C_WRTSWP, LUNL1C, MDS_TIME, ABS_ORBIT, ISWP, ALT, ALT_NOM, LAT, LON, $
        RAD_EARTH, NMWSWP, CLDRAD, CLDIDX

; Compute NESR for each microwindow
      L1C_MICNOI, NUM_NESR_PNTS, NESR_FIRST_WAVENUM, NESR_LAST_WAVENUM, $
        NESR_DATA, ISWP, NMIC, OUTMIC, WN1MIC, WN2MIC, NOIMIC
; Allow for reduction if reduced resln (NOIMIC only defined if NMIC>0)
      IF NMIC GT 0 THEN NOIMIC = NOIMIC * SQRT ( RESFAC )   

; Write out microwindow radiances
      L1C_WRTMIC, LUNL1C, SPEC, NMIC, OUTMIC, CODMIC, RESL1C, WN1MIC, $
        WN2MIC, NOIMIC

      ISWEEP = ISWEEP + 1
    ENDFOR                   ; End ISWP loop over sweeps within scan

    FREE_LUN, LUNL1C

  NEXT_SCAN:
  ENDFOR                     ; End ISCAN loop over scans

  FREE_LUN, LUNL1B

; If Batch mode, update FILSAV with latest file used
  IF BATCHMODE THEN BEGIN
    OPENW, LUNSAV, FILSAV, /GET_LUN
    PRINTF, LUNSAV, L1BFIL, FORMAT='(A)'
    FREE_LUN, LUNSAV   
  ENDIF

ENDWHILE

FREE_LUN, LUNLST

END
